package	jp.co.sds_corp.syslog.action;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.layout.ILayout;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;

/**
* 動作の基底クラスです。
* &lt;Valid&gt;要素をfalseに指定すると、無効になります。
* &lt;Layout&gt;要素と&lt;Filter&gt;要素はここに保持していますが
必須ではありません。
*/
public abstract class AbstractAction implements IAction
{
	private static final long serialVersionUID = 1L;
	ILayout	layout;
	boolean	valid = true;
	Vector<IFilter>	vecFilter = new Vector<IFilter>();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		try {
			setLayout( createLayout( config.getChild( "Layout" )));
		} catch ( UnsupportedOperationException uex ) {
		}
		valid = config.getBoolean( "Valid", true );

		Iterator<IConfiguration> itr;
		itr = config.getChilds( "Filter" );
		while ( itr.hasNext()) {
			IFilter filter = createFilter( (IConfiguration )itr.next() );
			vecFilter.add( filter );
		}
	}

	/**
	* syslogをフォーマットするIFilterを生成します。
	* @return	生成されたIFilter
	*/
	protected IFilter createFilter( IConfiguration config ) throws Exception {
		IFilter ret = (IFilter )ConfigUtility.createInstance( config );
		ret.init( config );
		return ret;
	}

	/**
	* syslogをフォーマットするILayoutを生成します。
	* @return	生成されたILayout
	*/
	protected ILayout createLayout( IConfiguration config ) throws Exception {
		if ( config == null ) {
			return null;
		}
		ILayout ret = (ILayout )ConfigUtility.createInstance( config );
		String fmt = config.getAttributes().get( "Format", null );
		if ( fmt == null ) {
			return null;
		}
		ret.setFormat( fmt );
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		IConfiguration c;
		config.setAttribute( "class", getClass().getName() );
		if ( layout != null ) {
			c = config.addChild( "Layout" );
			c.setAttribute( "class", layout.getClass().getName() );
			c.setAttribute( "Format", layout.getFormat() );
		}
		if ( !isValid()) {
			config.addChild( "Valid", "false" );
		}

		for ( int i = 0; i < vecFilter.size(); i++ ) {
			IFilter f = (IFilter )vecFilter.get( i );
			c = config.addChild( "Filter" );
			f.saveConfig( c );
		}
	}

	/**
	* 設定内容の文字列表現を取得します。
	*/                      
	public String paramString() {
		String ret = "";
		if ( !isValid()) {
			ret += "X ";
		}
		return ret;
	}

	/**
	* このアクションの文字列表現を返します。
	*/
	public String toString() {
		StringBuilder buf = new StringBuilder();
		buf.append( getClass().getName());
		if ( paramString() != null ) {
			buf.append( " " + paramString());
		}
		if ( vecFilter.size() > 0 ) {
			buf.append( " F{" );
			for ( int i = 0; i < vecFilter.size(); i++ ) {
				if ( i > 0 ) {
					buf.append( "," );
				}
				buf.append( vecFilter.get( i ).toString());
			}
			buf.append( "}" );
		}
		if ( layout != null ) {
			buf.append( " L{" + layout.toString() + "}" );
		}
		return buf.toString();
	}

	/**
	* 設定されているフィルタの一覧を返します。
	* @return	フィルタの一覧
	*/
	public List<IFilter> listFilters() {
		return vecFilter;
	}

	/**
	* フィルタを追加します。
	* @param	filter	設定するフィルタ
	*/
	public void addFilter( IFilter filter ) {
		vecFilter.add( filter );
	}

	/**
	* フィルタを追加します。
	* @param	list	設定するフィルタの格納されているList
	*/
	public void addFilterList( List<IFilter> list ) {
		vecFilter.addAll( list );
	}

	/**
	* フィルタをクリアします。
	*/
	public void clearFilter() {
		vecFilter.clear();
	}

	/**
	* この動作が有効であるかどうかを返します。
	*/
	public boolean isValid() {
		return valid;
	}

	/**
	* この動作の有効であるかどうかを設定します。
	* @param	var		有効フラグ
	*/
	public void setValid( boolean var ) {
		this.valid = var;
	}

	/**
	* syslogをフォーマットするILayout(layout)を設定します。
	* @param	var		設定するデータ
	*/
	public void setLayout( ILayout var ) {
		this.layout = var ;
	}
	/**
	* syslogをフォーマットするILayout(layout)を取得します。
	* @return	layoutの内容
	*/
	public ILayout getLayout() {
		return layout;
	}

}
