package	jp.co.sds_corp.syslog.filter;

import java.util.regex.Pattern;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* syslogのメッセージ内容によりフィルタリングを行ないます。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR>
		<TD>Pattern</TD>
		<TD>&nbsp;</TD>
		<TD>検索文字列を正規表現形式で指定</TD>
		<TD>&nbsp;</TD>
	</TR>
	<TR>
		<TD>IgnoreCase</TD>
		<TD>&nbsp;</TD>
		<TD>大文字小文字の違いを無視するか(true:無視する,false:区別する)</TD>
		<TD>指定されない場合はfalse</TD>
	</TR>
	<TR>
		<TD>Exclude</TD>
		<TD>&nbsp;</TD>
		<TD>除外フラグ
			(true:条件に合致すれば除外/false:条件に合致するもののみ対象)</TD>
		<TD>指定されない場合はfalse</TD>
	</TR>
	<TR>
		<TD>Valid</TD>
		<TD>&nbsp;</TD>
		<TD>有効フラグ(true:有効/false:無効)</TD>
		<TD>指定されない場合はtrue</TD>
	</TR>
</TABLE>
<PRE>
&lt;Filter class="jp.co.sds_corp.syslog.filter.PatternFilter"&gt;
	&lt;Pattern&gt;.*pop3.*&lt;/Pattern&gt;
	&lt;IgnoreCase&gt;true&lt;/IgnoreCase&gt;
	&lt;Exclude&gt;true&lt;/Exclude&gt;
	&lt;Valid&gt;false&lt;/Valid&gt;
&lt;/Filter&gt;
</PRE>
*/
public class PatternFilter extends AbstractFilter
{
	private static final long serialVersionUID = 1L;
	String	pattern;
	boolean	bIgnoreCase;

	/**
	* 指定された初期情報で初期化します。
	* @param    config  初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		String var;
		var = config.get( "Pattern", null );
		if ( var != null ) {
			pattern = var;
		}
		bIgnoreCase = config.getBoolean( "IgnoreCase", false );
	}

	/**
	* syslogのがこのフィルタの条件に合致するかを検査します。
	* @param	e		データ受信イベント
	*/
	public boolean match( SyslogEvent e ) {
		SyslogMsg msg = e.getMessage();
		if ( pattern != null ) {
			int flg = Pattern.DOTALL;
			if ( isIgnoreCase()) { 
				flg |=  Pattern.CASE_INSENSITIVE;
			}
			Pattern ptn = Pattern.compile( pattern, flg );
			if ( !ptn.matcher( msg.getMessage() ).matches() ) {
				return false;
			}
		}
		return true;
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = "{";
		ret += super.paramString();
		if ( pattern != null ) {
			ret += pattern;
		}
		if ( isIgnoreCase() ) {
			ret += " IGNORE_CASE";
		}
		ret += "}";
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "Pattern", getPattern() );
		if ( isIgnoreCase() ) {
			config.addChild( "IgnoreCase", "true" );
		}
	}

	/**
	* メッセージのパターン(pattern)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPattern( String var ) {
		this.pattern = var ;
	}
	/**
	* メッセージのパターン(pattern)を取得します。
	* @return	patternの内容
	*/
	public String getPattern() {
		return pattern;
	}

	/**
	* 大文字小文字を区別しない(ignoreCase)を設定します。
	* @param	var		設定するデータ
	*/
	public void setIgnoreCase( boolean var ) {
		this.bIgnoreCase = var ;
	}
	/**
	* 大文字小文字を区別しない(ignoreCase)を取得します。
	* @return	ignoreCaseの内容
	*/
	public boolean isIgnoreCase() {
		return bIgnoreCase;
	}


}
